from inspect import signature as createInspectSignature
from injecta.service.class_.InspectedArgumentResolver import InspectedArgumentResolver

def inspectFunction(obj):
    inspectedArgumentResolver = InspectedArgumentResolver()
    signature = createInspectSignature(obj)

    return list(map(lambda argument: inspectedArgumentResolver.resolve(argument[0], argument[1]), signature.parameters.items()))



from databricksbundle.dbutils.DbUtilsWrapper import DbUtilsWrapper
from databricksbundle.dbutils.IPythonDbUtilsResolver import resolveDbUtils

class DatabricksDbUtilsFactory:

    def create(self) -> DbUtilsWrapper:
        return DbUtilsWrapper(resolveDbUtils)



import re
from injecta.container.ContainerInterface import ContainerInterface
from pyfonybundles.Bundle import Bundle
from databricksbundle.notebook.NotebookErrorHandler import setNotebookErrorHandler
from databricksbundle.detector import isDatabricks
from databricksbundle.notebook.helpers import getNotebookPath, isNotebookEnvironment

class DatabricksBundle(Bundle):

    @staticmethod
    def autodetect():
        if isDatabricks():
            if isNotebookEnvironment():
                return DatabricksBundle('databricks_notebook.yaml')

            return DatabricksBundle('databricks_script.yaml')

        return DatabricksBundle('databricks_connect.yaml')

    def __init__(self, sparkConfigFilename: str):
        self.__sparkConfigFilename = sparkConfigFilename

    def getConfigFiles(self):
        return ['config.yaml', self.__sparkConfigFilename]

    def boot(self, container: ContainerInterface):
        parameters = container.getParameters()

        if (
            isDatabricks()
            and isNotebookEnvironment()
            and parameters.databricksbundle.enableNotebookErrorHandler is True
            and not re.match('^/Users/', getNotebookPath())
        ):
            setNotebookErrorHandler()



import IPython

def resolveDbUtils():
    ipython = IPython.get_ipython()

    if not hasattr(ipython, 'user_ns') or 'dbutils' not in ipython.user_ns:
        raise Exception('dbutils cannot be resolved')

    return ipython.user_ns['dbutils']



from pathlib import Path
import os

def setJavaHome():
    databricksJavaPath = Path.home().joinpath('.databricks-connect-java')

    if not databricksJavaPath.exists():
        raise Exception(f'Databricks Java (JDK) path {databricksJavaPath} does not exist')

    os.environ['JAVA_HOME'] = str(databricksJavaPath)



from typing import Tuple
from pyspark.sql import DataFrame

def transform(fun: callable, sources: Tuple[callable], arguments: tuple) -> DataFrame:
    g = fun.__globals__

    def transformSource(source: callable):
        return g[source.__name__ + '_df']

    dataframesToUse = tuple(map(transformSource, sources))

    df = fun(*(dataframesToUse + arguments))
    g[fun.__name__ + '_df'] = df

    return df



from typing import List
from pyspark.sql import SparkSession
from databricksbundle.spark.SparkSessionLazy import SparkSessionLazy
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface

class ScriptSessionFactory:

    def __init__(
        self,
        configurators: List[ConfiguratorInterface] = None,
    ):
        self.__configurators = configurators or []

    def create(self) -> SparkSessionLazy:
        def createLazy():
            spark = SparkSession \
                .builder \
                .getOrCreate()

            for configurator in self.__configurators:
                configurator.configure(spark)

            return spark

        return SparkSessionLazy(createLazy)



import IPython
import logging

# inspired by https://stackoverflow.com/questions/40110540/jupyter-magic-to-handle-notebook-exceptions
def setNotebookErrorHandler():
    def custom_exc(shell, etype, evalue, tb, tb_offset=None): # pylint: disable = invalid-name
        logging.error('Notebook exception', exc_info=True)

        # still show the error within the notebook, don't just swallow it
        shell.showtraceback((etype, evalue, tb), tb_offset=tb_offset)

    # this registers a custom exception handler for the whole current notebook
    IPython.get_ipython().set_custom_exc((Exception,), custom_exc)



import IPython
from databricksbundle.detector import isDatabricks

def createDisplay():
    if isDatabricks():
        return IPython.get_ipython().user_ns['display']

    from pyspark.sql.dataframe import DataFrame # pylint: disable = import-outside-toplevel
    import pandas as pd # pylint: disable = import-outside-toplevel

    pd.set_option('display.max_rows', 500)
    pd.set_option('display.max_columns', 500)
    pd.set_option('display.width', 1000)

    def displayLocal(df: DataFrame):
        print(df.limit(100).toPandas())

    return displayLocal

display = createDisplay()



import unittest
from injecta.testing.servicesTester import testServices
from injecta.config.YamlConfigReader import YamlConfigReader
from injecta.package.pathResolver import resolvePath
from typing import List
from loggerbundle.LoggerBundle import LoggerBundle
from pyfony.kernel.BaseKernel import BaseKernel
from pyfonybundles.Bundle import Bundle
from databricksbundle.DatabricksBundle import DatabricksBundle

class DatabricksBundleTest(unittest.TestCase):

    def test_azure(self):
        container = self.__createContainer('test_azure')

        testServices(container)

    def test_aws(self):
        container = self.__createContainer('test_aws')

        testServices(container)

    def test_test(self):
        container = self.__createContainer('test_test')

        testServices(container)

    def __createContainer(self, appEnv: str):
        class Kernel(BaseKernel):

            _allowedEnvironments = ['test_aws', 'test_azure', 'test_test']

            def _registerBundles(self) -> List[Bundle]:
                return [
                    LoggerBundle(),
                    DatabricksBundle('spark_test.yaml')
                ]

        kernel = Kernel(
            appEnv,
            resolvePath('databricksbundle') + '/_config',
            YamlConfigReader()
        )

        return kernel.initContainer()

if __name__ == '__main__':
    unittest.main()



from pathlib import Path
import os

def setHadoopHomeEnvVar():
    venvPath = os.getcwd() + '/.venv'

    hadoopHomePath = Path(Path(venvPath).joinpath('hadoop'))

    os.environ['HADOOP_HOME'] = str(hadoopHomePath)



from typing import Tuple

def saveDataFrame(fun: callable, sourceFunctions: Tuple[callable], arguments: tuple):
    g = fun.__globals__

    def transformSource(source: callable):
        return g[source.__name__ + '_df']

    dataframesToUse = tuple(map(transformSource, sourceFunctions))

    fun(*(dataframesToUse + arguments))



import types
import platform
from databricksbundle.detector import isDatabricks

class DbUtilsWrapper:

    def __init__(self, factoryCallback: callable):
        self._factoryCallback = factoryCallback
        self._dbUtils = None

    def __getattr__(self, attributeName):
        if self._dbUtils is None:
            if not isDatabricks():
                from databricksbundle.spark.javaHomeSetter import setJavaHome # pylint: disable = import-outside-toplevel
                from databricksbundle.spark.existingSparkCleanup import cleanExistingSparkConfig  # pylint: disable = import-outside-toplevel
                setJavaHome()
                cleanExistingSparkConfig()

                if platform.system() == 'Windows':
                    from databricksbundle.spark.hadoopHomeSetter import setHadoopHomeEnvVar # pylint: disable = import-outside-toplevel
                    setHadoopHomeEnvVar()

            self._dbUtils = self._factoryCallback()

        if hasattr(self._dbUtils, attributeName) is False:
            raise AttributeError(attributeName)

        attr = getattr(self._dbUtils, attributeName)

        if isinstance(attr, types.FunctionType) is False:
            return attr

        def wrapper(*args, **kw):
            return attr(*args, **kw)

        return wrapper



from pyspark.sql.session import SparkSession
from pyspark.sql.dataframe import DataFrame
from databricksbundle.jdbc.OptionsFactoryInterface import OptionsFactoryInterface

class TableReader:

    def __init__(
        self,
        optionsFactory: OptionsFactoryInterface,
        spark: SparkSession,
    ):
        self.__optionsFactory = optionsFactory
        self.__spark = spark

    def read(self, tableName: str, **kwargs) -> DataFrame:
        options = {**self.__optionsFactory.create(), **kwargs}

        return (
            self.__spark
                .read
                .format('jdbc')
                .options(**options)
                .option('dbtable', tableName)
                .load()
        )



import unittest
from databricksbundle.dbutils.DbUtilsWrapper import DbUtilsWrapper

class DbUtilsWrapperTest(unittest.TestCase):

    def test_objectAttributeDelegation(self):
        def createLazy():
            from pyspark.dbutils import DBUtils
            return DBUtils()

        dbUtilsWrapper = DbUtilsWrapper(createLazy)
        result = dbUtilsWrapper.fs.ls('/')

        self.assertIsInstance(result, list)

    def test_scalarAttributeDelegation(self):
        def createLazy():
            from pyspark.dbutils import DBUtils
            dbUtils = DBUtils()
            dbUtils.a = 5
            return dbUtils

        dbUtilsWrapper = DbUtilsWrapper(createLazy)
        result = dbUtilsWrapper.a

        self.assertEqual(5, result)

    def test_methodWithArgument(self):
        def createLazy():
            from pyspark.dbutils import FSHandler
            return FSHandler('foo')

        dbUtilsWrapper = DbUtilsWrapper(createLazy)
        result = dbUtilsWrapper.print_return(12345)

        self.assertEqual(12345, result)

    def test_delegationNonExistentAttribute(self):
        def createLazy():
            from pyspark.dbutils import DBUtils
            return DBUtils()

        dbUtilsWrapper = DbUtilsWrapper(createLazy)

        with self.assertRaises(AttributeError):
            dbUtilsWrapper.nonExistentMethod()

if __name__ == '__main__':
    unittest.main()



# pylint: disable = invalid-name
import sys
from pathlib import Path
from typing import Tuple, List
from injecta.service.class_.InspectedArgument import InspectedArgument
from databricksbundle.detector import isDatabricks
from databricksbundle.display import display
from databricksbundle.notebook.helpers import isNotebookEnvironment
from databricksbundle.pipeline.decorator.containerLoader import containerInitEnvVarDefined
from databricksbundle.pipeline.function.ArgumentsResolver import ArgumentsResolver
from databricksbundle.pipeline.decorator.argsChecker import checkArgs
from databricksbundle.pipeline.decorator.executor.dataFrameLoader import loadDataFrame
from databricksbundle.pipeline.decorator.executor.transformation import transform
from databricksbundle.pipeline.decorator.executor.dataFrameSaver import saveDataFrame
from databricksbundle.pipeline.function.functionInspector import inspectFunction

def _getContainer():
    if containerInitEnvVarDefined():
        from databricksbundle.container.envVarContainerLoader import container # pylint: disable = import-outside-toplevel
    else:
        from databricksbundle.container.pyprojectContainerLoader import container # pylint: disable = import-outside-toplevel

    return container

def _getPipelinePath():
    if isDatabricks() and not isNotebookEnvironment():
        if len(sys.argv) == 1:
            raise Exception('spark_python_task.parameters in Databricks job configuration must contain real pipeline path')

        return Path(sys.argv[1])

    return Path(sys.argv[0])

def _resolveArguments(inspectedArguments: List[InspectedArgument], decoratorArgs: tuple):
    argumentsResolver: ArgumentsResolver = _getContainer().get(ArgumentsResolver)
    return argumentsResolver.resolve(inspectedArguments, decoratorArgs, _getPipelinePath())

def _pipelineFunctionExecuted(fun):
    return fun.__module__ == '__main__'

def _resolveSourceFunctions(decoratorArgs: tuple) -> Tuple[callable]:
    sourceFunctions: List[callable] = []

    for decoratorArg in decoratorArgs:
        if callable(decoratorArg):
            sourceFunctions.append(decoratorArg)
        else:
            break

    return tuple(sourceFunctions)

class pipelineFunction:

    def __init__(self, *args): # pylint: disable = unused-argument
        self._decoratorArgs: tuple = args
        checkArgs(args, self.__class__.__name__)

    def __call__(self, fun, *args, **kwargs):
        if _pipelineFunctionExecuted(fun):
            arguments = _resolveArguments(inspectFunction(fun), self._decoratorArgs)
            fun(*arguments)

        return fun

class dataFrameLoader:

    def __init__(self, *args, **kwargs): # pylint: disable = unused-argument
        self._decoratorArgs: tuple = args
        self._displayEnabled = kwargs.get('display', False)
        checkArgs(args, self.__class__.__name__)

    def __call__(self, fun, *args, **kwargs):
        if _pipelineFunctionExecuted(fun):
            arguments = _resolveArguments(inspectFunction(fun), self._decoratorArgs)
            df = loadDataFrame(fun, arguments)

            if self._displayEnabled:
                display(df)

        return fun

class transformation:

    def __init__(self, *args, **kwargs): # pylint: disable = unused-argument
        self._decoratorArgs: tuple = args
        self._displayEnabled = kwargs.get('display', False)

    def __call__(self, fun, *args, **kwargs):
        if _pipelineFunctionExecuted(fun):
            sourceFunctions = _resolveSourceFunctions(self._decoratorArgs)
            startIndex = len(sourceFunctions)
            arguments = _resolveArguments(inspectFunction(fun)[startIndex:], self._decoratorArgs[startIndex:])
            df = transform(fun, sourceFunctions, arguments)

            if self._displayEnabled:
                display(df)

        return fun

class dataFrameSaver:

    def __init__(self, *args):
        self._decoratorArgs: tuple = args

    def __call__(self, fun, *args, **kwargs):
        if _pipelineFunctionExecuted(fun):
            sourceFunctions = _resolveSourceFunctions(self._decoratorArgs)
            startIndex = len(sourceFunctions)
            arguments = _resolveArguments(inspectFunction(fun)[startIndex:], self._decoratorArgs[startIndex:])
            saveDataFrame(fun, self._decoratorArgs[0], arguments)

        return fun



from pyspark.sql import SparkSession
from databricksbundle.dbutils.DbUtilsWrapper import DbUtilsWrapper

class DatabricksConnectDbUtilsFactory:

    def __init__(self, spark: SparkSession):
        self.__spark = spark

    def create(self) -> DbUtilsWrapper:
        def createLazy():
            from pyspark.dbutils import DBUtils # pylint: disable = import-outside-toplevel

            return DBUtils(self.__spark.sparkContext)

        return DbUtilsWrapper(createLazy)



from pathlib import Path
from typing import List
from injecta.service.class_.InspectedArgument import InspectedArgument
from databricksbundle.pipeline.function.ArgumentResolver import ArgumentResolver

class ArgumentsResolver:

    def __init__(
        self,
        argumentResolver: ArgumentResolver,
    ):
        self.__argumentResolver = argumentResolver

    def resolve(self, inspectedArguments: List[InspectedArgument], decoratorArgs: tuple, pipelinePath: Path):
        def resolve(functionArgument: InspectedArgument, decoratorArgument):
            return self.__argumentResolver.resolve(functionArgument, decoratorArgument, pipelinePath)

        decoratorArgsCount = len(decoratorArgs)

        if decoratorArgsCount > len(inspectedArguments):
            raise Exception('There are more decorator arguments than function arguments')

        return tuple(resolve(functionArgument, decoratorArgs[idx] if idx < decoratorArgsCount else None) for idx, functionArgument in enumerate(inspectedArguments))



from pathlib import Path
from typing import Dict
from injecta.container.ContainerInterface import ContainerInterface
from injecta.dtype.AbstractType import AbstractType
from injecta.dtype.classLoader import loadClass
from injecta.parameter.allPlaceholdersReplacer import replaceAllPlaceholders, findAllPlaceholders
from injecta.service.class_.InspectedArgument import InspectedArgument
from databricksbundle.pipeline.function.service.ServiceResolverInterface import ServiceResolverInterface

class ArgumentResolver:

    def __init__(
        self,
        serviceResolvers: Dict[str, str],
        container: ContainerInterface
    ):
        self.__serviceResolversMapping = serviceResolvers or []
        self.__container = container

    def resolve(self, functionArgument: InspectedArgument, decoratorArgument, pipelinePath: Path):
        argumentType = functionArgument.dtype

        if decoratorArgument is not None:
            if isinstance(decoratorArgument, str):
                output = self.__resolveStringArgument(decoratorArgument)
                return self.__checkType(output, argumentType, functionArgument.name)

            return self.__checkType(decoratorArgument, argumentType, functionArgument.name)

        if functionArgument.hasDefaultValue():
            return self.__checkType(functionArgument.defaultValue, argumentType, functionArgument.name)

        if not argumentType.isDefined():
            raise Exception(f'Argument "{functionArgument.name}" must either have explicit value, default value or typehint defined')

        argumentTypeStr = str(argumentType)

        if argumentTypeStr in self.__serviceResolversMapping:
            if self.__serviceResolversMapping[argumentTypeStr][0:1] != '@':
                raise Exception(f'Service name must start with @ for argument {functionArgument.name}')

            serviceResolverName = self.__serviceResolversMapping[argumentTypeStr][1:]
            serviceResolver: ServiceResolverInterface = self.__container.get(serviceResolverName)

            return serviceResolver.resolve(pipelinePath)

        class_ = loadClass(argumentType.moduleName, argumentType.className) # pylint: disable = invalid-name

        return self.__container.get(class_)

    def __resolveStringArgument(self, decoratorArgument):
        if decoratorArgument[0:1] == '@':
            return self.__container.get(decoratorArgument[1:])

        matches = findAllPlaceholders(decoratorArgument)

        if not matches:
            return decoratorArgument

        parameters = self.__container.getParameters()

        return replaceAllPlaceholders(decoratorArgument, matches, parameters, decoratorArgument)

    def __checkType(self, value, expectedType: AbstractType, argumentName: str):
        valueTypeStr = value.__class__.__module__ + '.' + value.__class__.__name__
        expectedTypeStr = str(expectedType)

        if expectedType.isDefined() and valueTypeStr != expectedTypeStr:
            expectedTypeStr = expectedTypeStr.replace('builtins.', '')
            valueTypeStr = valueTypeStr.replace('builtins.', '')
            raise Exception(f'Argument "{argumentName}" is defined as {expectedTypeStr}, {valueTypeStr} given instead')

        return value



import os
from injecta.dtype.classLoader import loadClass
from injecta.container.ContainerInterface import ContainerInterface

def loadContainerUsingEnvVar(appEnv: str) -> ContainerInterface:
    if 'CONTAINER_INIT_FUNCTION' not in os.environ:
        raise Exception('CONTAINER_INIT_FUNCTION environment variable must be set on cluster')

    containerInitFunctionPath = os.environ['CONTAINER_INIT_FUNCTION']
    containerInitModuleName = containerInitFunctionPath[0:containerInitFunctionPath.rfind('.')]
    containerInitFunctionName = containerInitFunctionPath[containerInitFunctionPath.rfind('.') + 1:]

    containerInitFunction = loadClass(containerInitModuleName, containerInitFunctionName)
    return containerInitFunction(appEnv)

def containerInitEnvVarDefined():
    return 'CONTAINER_INIT_FUNCTION' in os.environ



import logging
import unittest
from pathlib import Path
from box import Box
from injecta.container.ContainerInterface import ContainerInterface
from injecta.dtype.DType import DType
from injecta.service.class_.InspectedArgument import InspectedArgument
from databricksbundle.pipeline.function.ArgumentResolver import ArgumentResolver
from databricksbundle.pipeline.function.service.ServiceResolverInterface import ServiceResolverInterface
from databricksbundle.spark.ScriptSessionFactory import ScriptSessionFactory

class ArgumentResolverTest(unittest.TestCase):

    def setUp(self):
        serviceResolvers = {
            'logging.Logger': '@databricksbundle.test.DummyLoggerResolver'
        }

        self.__argumentResolver = ArgumentResolver(serviceResolvers, self.__createDummyContainer())

    def test_explicitIntArgument(self):
        inspectedArgument = InspectedArgument('myVar', DType('builtins', 'int'))

        resolvedArgument = self.__argumentResolver.resolve(inspectedArgument, 123, Path('.'))

        self.assertEqual(123, resolvedArgument)

    def test_plainStrArgument(self):
        inspectedArgument = InspectedArgument('myVar', DType('builtins', 'str'))

        resolvedArgument = self.__argumentResolver.resolve(inspectedArgument, 'Hello', Path('.'))

        self.assertEqual('Hello', resolvedArgument)

    def test_strArgumentWithPlaceholders(self):
        inspectedArgument = InspectedArgument('myVar', DType('builtins', 'str'), 'Some default hello', True)

        resolvedArgument = self.__argumentResolver.resolve(inspectedArgument, 'Hello %name% %surname%', Path('.'))

        self.assertEqual('Hello Peter Novak', resolvedArgument)

    def test_strArgumentService(self):
        inspectedArgument = InspectedArgument('myVar', DType(ScriptSessionFactory.__module__, 'ScriptSessionFactory'))

        resolvedSparkSessionFactory = self.__argumentResolver.resolve(inspectedArgument, f'@{ScriptSessionFactory.__module__}', Path('.'))

        self.assertIsInstance(resolvedSparkSessionFactory, ScriptSessionFactory)

    def test_argumentWithDefaultValue(self):
        inspectedArgument = InspectedArgument('myVar', DType('builtins', 'str'), 'Peter', True)

        resolvedArgument = self.__argumentResolver.resolve(inspectedArgument, None, Path('.'))

        self.assertEqual('Peter', resolvedArgument)

    def test_noValueNoTypehint(self):
        inspectedArgument = InspectedArgument('myVar', DType('inspect', '_empty'))

        with self.assertRaises(Exception) as error:
            self.__argumentResolver.resolve(inspectedArgument, None, Path('.'))

        self.assertEqual('Argument "myVar" must either have explicit value, default value or typehint defined', str(error.exception))

    def test_mappedService(self):
        inspectedArgument = InspectedArgument('myLogger', DType('logging', 'Logger'))

        resolvedLogger = self.__argumentResolver.resolve(inspectedArgument, None, Path('.'))

        self.assertIsInstance(resolvedLogger, logging.Logger)
        self.assertEqual('test_logger', resolvedLogger.name)

    def test_generalService(self):
        inspectedArgument = InspectedArgument('sparkSessionFactory', DType(ScriptSessionFactory.__module__, 'ScriptSessionFactory'))

        resolvedSparkSessionFactory = self.__argumentResolver.resolve(inspectedArgument, None, Path('.'))

        self.assertIsInstance(resolvedSparkSessionFactory, ScriptSessionFactory)

    def __createDummyContainer(self):
        class DummyLoggerResolver(ServiceResolverInterface):

            def resolve(self, pipelinePath: Path) -> object:
                return logging.getLogger('test_logger')

        class DummyContainer(ContainerInterface):

            def getParameters(self) -> Box:
                return Box({
                    'name': 'Peter',
                    'surname': 'Novak'
                })

            def get(self, ident):
                if ident == 'databricksbundle.test.DummyLoggerResolver':
                    return DummyLoggerResolver()

                if ident == ScriptSessionFactory.__module__ or ident.__module__ == ScriptSessionFactory.__module__:
                    return ScriptSessionFactory()

                raise Exception(f'Unexpected service {ident}')

        return DummyContainer()

if __name__ == '__main__':
    unittest.main()



from typing import List
from pyspark.sql import SparkSession # pylint: disable = unused-import
from databricksbundle.spark.SparkSessionLazy import SparkSessionLazy
import IPython
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface

class NotebookSessionFactory:

    def __init__(
        self,
        configurators: List[ConfiguratorInterface],
    ):
        self.__configurators = configurators

    def create(self) -> SparkSessionLazy:
        spark = IPython.get_ipython().user_ns['spark'] # type: SparkSession

        for configurator in self.__configurators:
            configurator.configure(spark)

        IPython.get_ipython().user_ns['spark'] = spark

        return SparkSessionLazy(lambda: spark)



import sys
from databricksbundle.dbutils.IPythonDbUtilsResolver import resolveDbUtils

def getNotebookContext():
    return resolveDbUtils().notebook.entry_point.getDbutils().notebook().getContext()

def getUserEmail():
    return getNotebookContext().tags().get('user').get()

def getNotebookPath():
    return getNotebookContext().notebookPath().get()

def isNotebookEnvironment():
    return sys.argv and sys.argv[0][-15:] == '/PythonShell.py'



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['databricksbundle']

package_data = \
{'': ['*'],
 'databricksbundle': ['_config/*',
                      'container/*',
                      'dbutils/*',
                      'jdbc/*',
                      'notebook/*',
                      'pipeline/decorator/*',
                      'pipeline/decorator/executor/*',
                      'pipeline/function/*',
                      'pipeline/function/service/*',
                      'spark/*',
                      'spark/config/*']}

install_requires = \
['injecta>=0.8.9,<0.9.0',
 'ipython>=7.0.0,<8.0.0',
 'logger-bundle>=0.5.0,<0.6.0',
 'pandas>=0.24.2',
 'pyarrow>=0.16.0,<0.17.0',
 'pyfony-bundles>=0.2.0,<0.3.0']

setup_kwargs = {
    'name': 'databricks-bundle',
    'version': '0.5.7b1',
    'description': 'Databricks bundle for the Pyfony framework',
    'long_description': 'Databricks bundle for the Pyfony framework\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bricksflow/databricks-bundle',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<3.8.0',
}


setup(**setup_kwargs)



from pathlib import Path
from injecta.container.ContainerInterface import ContainerInterface
from databricksbundle.pipeline.function.service.ServiceResolverInterface import ServiceResolverInterface

class PipelineParamsResolver(ServiceResolverInterface):

    def __init__(
        self,
        container: ContainerInterface,
    ):
        self.__container = container

    def resolve(self, pipelinePath: Path):
        pipelineName = f'{pipelinePath.parent.parent.stem}.{pipelinePath.parent.stem}'
        parameters = self.__container.getParameters()

        if 'datalakebundle' not in parameters:
            raise Exception('Install and activate datalake-bundle to use pipeline params')

        if pipelineName not in parameters.datalakebundle.tables:
            raise Exception(f'Pipeline {pipelineName} is not defined in datalakebundle.tables')

        if 'params' not in parameters.datalakebundle.tables[pipelineName]:
            raise Exception(f'No pipeline params defined for {pipelineName} in datalakebundle.tables')

        return parameters.datalakebundle.tables[pipelineName].params



from pathlib import Path
from loggerbundle.LoggerFactory import LoggerFactory
from databricksbundle.pipeline.function.service.ServiceResolverInterface import ServiceResolverInterface

class LoggerResolver(ServiceResolverInterface):

    def __init__(
        self,
        loggerFactory: LoggerFactory,
    ):
        self.__loggerFactory = loggerFactory

    def resolve(self, pipelinePath: Path):
        pipelineName = f'{pipelinePath.parent.parent.stem}.{pipelinePath.parent.stem}'

        return self.__loggerFactory.create(pipelineName)



import platform
from databricksbundle.detector import isDatabricks

class SparkSessionLazy:

    def __init__(self, factoryCallback: callable):
        self._factoryCallback = factoryCallback
        self._sparkSession = None

    def __getattr__(self, attributeName):
        if self._sparkSession is None:
            if not isDatabricks():
                from databricksbundle.spark.javaHomeSetter import setJavaHome # pylint: disable = import-outside-toplevel
                from databricksbundle.spark.existingSparkCleanup import cleanExistingSparkConfig  # pylint: disable = import-outside-toplevel
                setJavaHome()
                cleanExistingSparkConfig()

                if platform.system() == 'Windows':
                    from databricksbundle.spark.hadoopHomeSetter import setHadoopHomeEnvVar # pylint: disable = import-outside-toplevel
                    setHadoopHomeEnvVar()

            self._sparkSession = self._factoryCallback()

        return getattr(self._sparkSession, attributeName)



from typing import Optional, List
from pyspark.sql.session import SparkSession
from pyspark.conf import SparkConf
from databricksbundle.spark.SparkSessionLazy import SparkSessionLazy
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface

class DatabricksConnectSessionFactory:

    def __init__(
        self,
        address: str,
        token: str,
        clusterId: str,
        orgId: Optional[str],
        port: int,
        bindAddress: Optional[str],
        configurators: List[ConfiguratorInterface],
    ):
        self.__address = address
        self.__token = token
        self.__clusterId = clusterId
        self.__orgId = orgId
        self.__port = port
        self.__bindAddress = bindAddress
        self.__configurators = configurators

    def create(self) -> SparkSessionLazy:
        if not self.__address:
            raise Exception('Databricks workspace address not set')

        if not self.__token:
            raise Exception('Databricks workspace token not set')

        if not self.__clusterId:
            raise Exception('Databricks cluster not set')

        if not self.__port:
            raise Exception('Databricks connect port not set')

        def createLazy():
            # Databricks Connect configuration must be set before calling getOrCreate()
            conf = SparkConf()
            conf.set('spark.databricks.service.address', self.__address)
            conf.set('spark.databricks.service.token', self.__token)
            conf.set('spark.databricks.service.clusterId', self.__clusterId)

            if self.__orgId is not None:
                conf.set('spark.databricks.service.orgId', self.__orgId)

            conf.set('spark.databricks.service.port', self.__port)

            if self.__bindAddress is not None:
                conf.set('spark.driver.bindAddress', self.__bindAddress)

            spark = SparkSession.builder.config(conf=conf).getOrCreate()

            for configurator in self.__configurators:
                configurator.configure(spark)

            return spark

        return SparkSessionLazy(createLazy)



from types import FunctionType

def checkArgs(a, decoratorName: str):
    # called as @[decoratorName] (without brackets and any arguments)
    if a and isinstance(a[0], FunctionType):
        raise Exception(f'Use @{decoratorName}() instead of @{decoratorName} please')
